#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "getopt.h"

#ifndef lint
static char rcsid[]="$Id: parse_option.c,v 2.14 1999/08/04 19:47:55 thomas Exp thomas $";
#endif

#define VER "2.0.4"

int make_connect(void);
void opt(int, char **);
/* Flag set by `--proxy'. */
static int proxy_flag = 0;
/* Flag set by `--wildcard'. */
static int wildcard_flag = 0;
/* Flag set by `--debug'. */
static int debug_flag = 0;
/* Flag set by `--help'. */
static int help_flag = 0;
/* Flag set by `--backmx'. */
static int backmx_flag = 0;
/* Vars for other options */
extern char *ip_address;
char *host_id;
char *mx_addr;
char *proxy_server;
char *proxy_port_ch;
int proxy_port = 0;
static int mx_flag = 0;

void opt(int argc, char **argv) {
	int c;

	while (1) {
		static struct option long_options[] = {
			/* These options set a flag. */
			{"wildcard", 0, &wildcard_flag, 1},
			{"help", 0, &help_flag, 1},
			{"debug", 0, &debug_flag, 1},
			{"backmx", 0, &backmx_flag, 1},
			/* These options don't set a flag.
			We distinguish them by their indices. */
			{"ip", 1, 0, 0},
			{"host", 1, 0, 0},
			{"mx", 1, 0, 0},
			{"proxyserv", 1, 0, 0},
			{"proxyport", 1, 0, 0},
			{0, 0, 0, 0} };
		/* getopt_long stores the option index here. */
		int option_index = 0;

		c = getopt_long (argc, argv, "f:", long_options, &option_index);

		/* Detect the end of the options. */
		if (c == -1)
			break;

		switch (c) {
 			case 0:
				/* If this option set a flag, do nothing else now. */
				if (long_options[option_index].flag != 0)
					break;
				if (long_options[option_index].name == "ip")
					ip_address = optarg;
				if (long_options[option_index].name == "mx")
					mx_addr = optarg;
				if (long_options[option_index].name == "host")
					host_id = optarg;
				if (long_options[option_index].name == "proxyserv")
					proxy_server = optarg;
				if (long_options[option_index].name == "proxyport") {
					proxy_port_ch = optarg;
					proxy_port = atoi(proxy_port_ch);
				}
				break;
			case '?':
				/* getopt_long already printed an error message. */
				break;
			default:
				abort ();
		}
	}

	if (proxy_server != NULL) {
		proxy_flag = 1;
	}

	if (help_flag) {
		usage();
	}

	if (host_id == NULL) {
		printf("--host is required to update.\n");
		usage();
	}
	
	if (mx_addr != NULL)
		mx_flag = 1;

	/* Print any remaining command line arguments (not options). */
	if (optind < argc) {
		printf ("Unknown non-option ARGV-elements: ");
		while (optind < argc)
			printf ("%s ", argv[optind++]);
		printf("We will try to continue\n");
	} 
}

int make_connect(void) {
   struct hostent *ptrh;
   struct protoent *ptrp;
   struct sockaddr_in sad;
   int sd;
   int port = 80;
   char *host = "members.dyndns.org";

   if(proxy_flag) {
	host = proxy_server;
	if(proxy_port != 0)
		port = proxy_port;
   }
   memset((char *)&sad,0,sizeof(sad));
   sad.sin_family = AF_INET;

   sad.sin_port = htons((u_short)port);
   
   ptrh = gethostbyname(host);

   if ( ((char *)ptrh) == NULL ) {
      fprintf(stderr,"invalid host: %s\n", host);
      exit(1);
   }

   memcpy(&sad.sin_addr, ptrh->h_addr, ptrh->h_length);

   if ( ((int)(ptrp = getprotobyname("tcp"))) == 0 ) {
      fprintf(stderr,"cannot map \"tcp\" to protocol number\n");
      exit(1);
   }

   sd = socket(PF_INET, SOCK_STREAM, ptrp->p_proto);
   
   if (sd < 0) {
      fprintf(stderr, "Socket creation failed\n");
      exit(1);
   }

   if (connect(sd, (struct sockaddr *)&sad, sizeof(sad)) < 0) {
      fprintf(stderr, "Connect failed\n");
      exit(1);
   }

   return(sd);
}

extern char auth_en[100];
extern char auth[100];
extern char hostname[100];
extern char pass[100];

void *safe_malloc(int n) {
   char *mem = malloc(n);
   if (!mem) {
     fprintf(stderr, "no memory available\n");
     exit(1);
   }
   return mem;
}
void *safe_realloc(void *orig, int n) {
   char *mem = realloc(orig, n);
   if (!mem) {
     fprintf(stderr, "no memory available\n");
     exit(1);
   }
   return mem;
}

int update(void) {
   char buf_in[1024];
   char buf_out[1000];
   int  response_size;
   char *response;
   char useragent[]="ddup/" VER;
   char *base_get = "GET /nic/dyndns?action=edit&started=1&hostname=YES&host_id=";
   int sd;	
   int result;

   if (proxy_flag) {
	base_get = "GET http://members.dyndns.org/nic/dyndns?action=edit&started=1&hostname=YES&host_id=";
   }

   sd = make_connect();

   buf_out[0] = '\0';
   strcat(buf_out, base_get);
   strcat(buf_out, host_id);
   if(ip_address) {
      strcat(buf_out, "&myip=");
      strcat(buf_out, ip_address);
   }
   if(wildcard_flag) {
      strcat(buf_out, "&wildcard=ON");
   }
   if(mx_flag) {
      strcat(buf_out, "&mx=");
      strcat(buf_out, mx_addr);
      if(backmx_flag) {
         strcat(buf_out, "&backmx=YES");
      }
   }
   strcat(buf_out, " HTTP/1.1\r\n");
   strcat(buf_out, "Host: members.dyndns.org\r\n");
   strcat(buf_out, "Authorization: Basic ");
   strcat(buf_out, auth_en);
   strcat(buf_out, "\r\n");
   strcat(buf_out, "Content-Type: application/x-www-form-urlencoded\r\n");
   strcat(buf_out, "User-Agent: ");
   strcat(buf_out, useragent);
   strcat(buf_out, "\r\n\r\n");
   if(debug_flag) {
      printf("%s\n", buf_out);
   }
   send(sd, buf_out, strlen(buf_out), 0);

   /* read the response back. start off with a NUL */
   response_size = 0;
   response      = safe_malloc(response_size+1);
   *response     = '\0';

   /* wait at most 60 seconds for the server to respond */
   alarm(60);
   do {
      buf_in[0] = '\0';
      result = recv(sd, buf_in, sizeof(buf_in), 0);

      if (result > 0) {
        /* append the part of the page read */
        response_size += result;
        response       = safe_realloc(response, response_size+1);
        buf_in[result] = '\0';
        strcat(response, buf_in);
        
	if (strstr(response, "</HTML>"))
	    break;
      }
   } while (result >= 0);

   if (debug_flag) {
      printf("%s\n", response);
   }
   if ((result > 0) && !strstr(response, "NOERROR\n")) {
      result = -1;
   }
   printf("Update %s\n", (result <= 0) ? "Failed" : "Succeeded");
   if (strstr(response, "blocked due to abuse")) {
      printf("Due to abuse.\n");
      printf("If this is an error contact support@dyndns.org and explain your problem.\n");
   }
   close(sd);

   return result <= 0;
}

